/**** ANAHEIM AUTOMATION ***
   Filename: CLCI500.C
   Programmer: Alan Harmon
   Date: 1/15/97
   Time: 3:30pm
   Revised by: Cody Brandel
   Date: 3/1/99

   Version: 2.02

   Version 2.01 changes
        include adding GET_INPUT, and correcting timing for faster computers
        and defining the function MOTION
   Version 2.02 changes
        added JOG function
   Version 3.0  includes a function for GetInputs, and streamlining old code.
        Also the code is seperated into a header, source code format.
        There is also an Initialize function,which will put the board in a
        known state on power up.
        This function will also "unlock" the board in case of failure during
        execution.
        Each function will now return a value, for error detection at any phase
        in the program.
        There is also a new function which tells if there is 1,2, or a two axis
        board with one axis not functioning.
        The above function uses two more new functions which are GetASCII,
        and GetBinary. Get Binary return 2 bytes.

   This is the .cpp file for the Computer Indexer Board CLCI500
*/

#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include "CLCI500.H"

int PortAddress = 0x300;

result ReceiveAscii(int Axis, byte  *Value);
result ReceiveNumber (int  Axis, unsigned int  *N);
result SendAscii(int Axis, byte  Command);
result SendNumber(int Axis, unsigned int  Number);

enum PortAddressOffsets
{
   AxisStatus, AxesControl, ReadAxisA, WriteAxisA, ReadAxisB,
   WriteAxisB, ReadInputs, WriteOutputs
};


int AxisBusy(int Axis)
{

   int a = inportb(PortAddress);

   if (Axis==1)
   {
     return (a & 1 ? 0 : 1); // 0 means NOT BUSY, 1 means BUSY
   }
   else
   {
     return (a & 4 ? 0 : 1); // 0 means NOT BUSY, 1 means BUSY
   }
}

result FinishMove(int Axis, unsigned int Duration)
{
   int TimeOutClock = 0;

  	while (AxisBusy(Axis))  // Stay here while the axis is busy
	  {
       if(Duration)
       {
         delay(1);
         if(inc(TimeOutClock) > Duration)
           Failure();
       }
	  }
	Success();
}

int GetInput(int InputBit)     // returns 0 if input is low and 1 if input is TTL high
{
   int BitValue;

   BitValue = inportb(PortAddress + ReadInputs);

   return ((BitValue >> InputBit - 1) & 0x01);
}

int GetInputs(void)               //This gets the integer sum of all inputs
{
   return (inportb(PortAddress + ReadInputs));
}

result GoAbsolute(int Axis, int Value)
{
	FailIf(SendAscii(Axis, 'P'));
	FailIf(SendNumber(Axis, Value));
   FailIf(SendAscii(Axis,'G'));
	Success();
}

int GetNumberOfAxes(void)
{
   const int CheckValue1 = 12;
   const int CheckValue2 = 21;
   int Axis;
   int AxisCount;
   unsigned int NValue;

   outportb (PortAddress + AxesControl, 7);
   /// cycle thru possible send/receive combinations

   Axis = 1;
   if (SendAscii(Axis,'N') or
       SendNumber(Axis,CheckValue1) or
       SendAscii(Axis,'V') or
       SendAscii(Axis,'N') or
       ReceiveNumber (Axis, &NValue))
   { // Axis 1 handshaking failed, therefore axis nonexistent
       AxisCount = 0;
   }
   else
   { // Axis responded properly
       AxisCount = (NValue == CheckValue1 ? 1 : 0);
   }

   Axis = 2;
   if (SendAscii(Axis,'N') or
       SendNumber(Axis,CheckValue2) or
       SendAscii(Axis,'V') or
       SendAscii(Axis,'N') or
       ReceiveNumber (Axis, &NValue))
   { // Axis 2 handshaking failed, therefore axis nonexistent
   }
   else
   { // Axis 2 responded properly
       if((AxisCount == 1) and (NValue == CheckValue2))
           AxisCount = 2;
       if((AxisCount == 0) and (NValue == CheckValue2))
           AxisCount = 99;
   }
   return(AxisCount);
}

result GoHome(int Axis, int Type)
{
	FailIf(SendAscii(Axis, 'H'));
	FailIf(SendNumber(Axis, Type));
	Success();
}

result GoRelative(int Axis)
{
	FailIf(SendAscii(Axis, 'G'));
	Success();
}

result InitializeAxis(int Axis)
{
   /// verify selected axis does exist

   int WriteDataAddress = (Axis == 1 ? (PortAddress +3):(PortAddress + 5));
   int AxisRequestMask = (Axis == 1 ? 4 : 2);
   int i;

   for (i = 0; i < 3; i++)
   {
      outportb(PortAddress + AxesControl,WriteOutputs);
         delay(500);
      outportb(WriteDataAddress, 'Q');
         delay(500);
      outportb(PortAddress + AxesControl, AxisRequestMask);
         delay(500);
      outportb(PortAddress + AxesControl, WriteOutputs);
         delay(500);
   }

	FailIf(SetAccel(Axis, 10));
	FailIf(SetDecel(Axis, 10));
	FailIf(SetMotion(Axis, 400, 1000)); // base of 400 max of 1000
	FailIf(SetDistance(Axis, 400));
	FailIf(SetCWDir(Axis));
   FailIf(SetOutputs(0,0,0,0,0,0));    // Turn off all outputs
   FailIf(SetJogDivisor(Axis,16));
   FailIf(ZeroPosition(Axis));
   Success();
}

result SetAccel(int Axis, int Value)
{
	FailIf(SendAscii(Axis, 'A'));
	FailIf(SendNumber(Axis, Value));
   Success();
}

result SetBase(int Axis, int Value)
{
	FailIf(SendAscii(Axis, 'B'));
	FailIf(SendNumber(Axis, Value));
	Success();
}

result SetCCWDir(int Axis)
{
	FailIf(SendAscii(Axis, '-'));
	Success();
}

result SetCWDir(int Axis)
{
	FailIf(SendAscii(Axis, '+'));
	Success();
}


result SetDecel(int Axis, int Value)
{
	FailIf(SendAscii(Axis, 'D'));
	FailIf(SendNumber(Axis, Value));
	Success();
}

result SetDistance(int Axis, int Distance)
{
   FailIf(SendAscii(Axis,'N'));
   FailIf(SendNumber(Axis,Distance));
   Success();
}

result SetFactor(int Axis, int Value)
{
	FailIf(SendAscii(Axis, 'F'));
	FailIf(SendNumber(Axis, Value));
	Success();
}


result SetJogDivisor(int Axis, int Value)  //Sets jog speed via divisor(Base/Divisor)
{
	FailIf(SendAscii(Axis, 'J'));
	FailIf(SendNumber(Axis, Value));
	Success();
}


result SetMax(int Axis, int Value)
{
	FailIf(SendAscii(Axis, 'M'));
	FailIf(SendNumber(Axis, Value));
	Success();
}


result SetMotion(int Axis, int Base, int Max)
{				// Sets the base speed, max speed and factor
	const double Crystal = 11000000.0;

	int Afactor;
	int MaxRate;
	int BaseRate;

   if(Base > 20000)
     Base = 20000;

   if(Base < 5)
     Base = 5;

   if(Max > 24444)
     Max = 24444;

   if(Max < 100)
     Max = 100;

	Afactor = Crystal / (Max * ( 8100.0 - (30.0 * 255.0)));

   if(Afactor < 0)
     Afactor = 0;

   if(Afactor > 255)
     Afactor = 255;

	MaxRate = ((8100.0 - Crystal / (Max * Afactor)) / 30.0 + 0.5);

	BaseRate = ((8100.0 - Crystal / (Base * Afactor)) / 30.0 + 0.5);

   if(BaseRate < 1)
     BaseRate = 1;

   if(BaseRate > 254)
     BaseRate = 254;

	FailIf(SetBase(Axis, BaseRate));
	FailIf(SetMax (Axis, MaxRate));
	FailIf(SetFactor(Axis, Afactor));
	Success();
}

int SetOutputs(int out6, int out5, int out4,
					int out3, int out2, int out1)   //Turn on any output by writing a 1 to it
{
   int Total = ((out6<<5) + (out5<<4) + (out4<<3) + (out3<<2) + (out2<<1) + (out1));
   outportb (PortAddress + WriteOutputs, Total);
   Success();
}

result ZeroPosition(int Axis)
{
	FailIf(SendAscii(Axis, 'Z'));
	Success();
}

result SendAscii(int Axis, byte Command)
{
	 int WriteDataAddress = PortAddress + (Axis == 1 ? 3 : 5);
    int AxisRequestMask =  (Axis == 1 ? 4 : 2);
    int TimeOutClock = 0;
    const int TimeOutDelay = 1000; // 1 second delay

    while(AxisBusy(Axis))
    {
      delay(1);
      if(inc(TimeOutClock) > TimeOutDelay)
        Failure();      //return -1 if chraracter didn't go through
    }

	    outportb(PortAddress + AxesControl,6);       // sets A:i/o req to HI, B:i/o req to HI, write control to LOW
	    outportb(WriteDataAddress, Command);   // latches the data on U7
	    outportb(PortAddress + AxesControl, AxisRequestMask);    // this set the A:i/o req to LOW, B:i/o req to HI, write control to LOW

       TimeOutClock = 0;
	    until(AxisBusy(Axis)) //wait until the chip is busy or 1 second passes, whichever comes first
	       {
             delay(1);
             if(inc(TimeOutClock) > TimeOutDelay)
                Failure();      //return -1 if chip takes longer than 1 second
          }

	    outportb(PortAddress + AxesControl, 7);  // this sets the A:i/o req to HI, B:i/o req to HI, write control to HI

       Success();        //return 0 ifcharacter went through ok
}


result SendNumber(int Axis, unsigned int Number)
{
    FailIf(SendAscii(Axis ,Number & 0xff));
    FailIf(SendAscii(Axis, (Number >> 8) & 0xff));
    Success();
}

result ReceiveAscii (int  Axis, byte  *Value)
{
    int  DBCheck;

    while(AxisBusy(Axis))
    {
    }

    outportb (PortAddress + 1, Axis == 1 ? 5 : 3);
    do
    {
        DBCheck = inport(PortAddress) & ((Axis == 1) ? 3 : 12);
    }
    until (DBCheck == 0);
    *Value = inportb(PortAddress + (Axis == 1 ? 2 : 4));
    outportb (PortAddress + 1, 7);
    do
    {
        DBCheck = inport(PortAddress) & ((Axis == 1) ? 1 : 4);
    }
    while (DBCheck == 0);
    Success ();
}

result ReceiveNumber (int  Axis, unsigned int  *N)
{
    byte  LSByte, MSByte;

    FailIf (ReceiveAscii (Axis, &LSByte));
    FailIf (ReceiveAscii (Axis, &MSByte));
    *N = (((unsigned int) MSByte) << 8) + LSByte;
    Success ();
}
